/*  MemoReader Ŀ
 *   Description: A Custom Reader to perform a MemoEdit() when the
 *                spacebar is hit (for Clipper 5.2x)              
 *     File Name: MEMOREAD.PRG                                    
 *        Author: Shalom LeVine                                   
 *     Copyright: (c) 1996 by Shalom LeVine                       
 *                        placed in the Public Domain             
 * */

#include "inkey.ch"
#include "Getexit.ch"
#define K_UNDO                 K_CTRL_U

#define CARGO_MEMO             1
#define CARGO_TOP              2
#define CARGO_LEFT             3
#define CARGO_BOTTOM           4
#define CARGO_RIGHT            5
#define CARGO_EDITMODE         6
#define CARGO_USERFUNC         7
#define CARGO_LINELEN          8
#define CARGO_TABSIZE          9
#define CARGO_TEXTBUFFERROW   10
#define CARGO_TEXTBUFFERCOL   11
#define CARGO_WINDOWROW       12
#define CARGO_WINDOWCOL       13
#define CARGO_MAXLEN          13

/*  Procedure Ŀ
 *          Name: MemoReader()                                   
 *   Description: The GetReader replacement for the Memoget      
 *        Author: Shalom LeVine         Designer: Shalom LeVine  
 *     Copyright: Shalom LeVine                                  
 * Ĵ
 *    Parameters: oGet                                           
 * */

PROCEDURE MemoReader( oGet )

    // Fill out the MEMOGET cargo array to the maximum size
    // so we don't get an ARRAY BOUND error when we call MemoEdit() later
    IF LEN( oGet:cargo ) < CARGO_MAXLEN
       ASIZE( oGet:cargo, CARGO_MAXLEN )
    END

   // Read the GET if the WHEN condition is satisfied
   IF ( GetPreValidate( oGet ) )

      // Activate the GET for reading
      oGet:setFocus()

      WHILE ( oGet:exitState == GE_NOEXIT )

             // Check for initial typeout (no editable positions)
         IF ( oGet:typeOut )
            oGet:exitState := GE_ENTER
         ENDIF

         // Apply keystrokes until exit
         WHILE ( oGet:exitState == GE_NOEXIT )
                MemoApplyKey( oGet, Inkey(0) )
         ENDDO

         // Disallow exit if the VALID condition is not satisfied
         IF ( !GetPostValidate( oGet ) )
            oGet:exitState := GE_NOEXIT
         ENDIF
      ENDDO

      // De-activate the GET
      oGet:killFocus()

   ENDIF

   RETURN

/*  Procedure Ŀ
 *          Name: MemoApplyKey()                                   
 *   Description: GetApplyKey replacement for the MemoGet          
 *        Author: Shalom LeVine         Designer: Shalom LeVine    
 *     Copyright: Shalom LeVine                                    
 * Ĵ
 *    Parameters: oGet                                             
 *              : nKey                                             
 * */
PROCEDURE MemoApplyKey( oGet, nKey )

   LOCAL cKey
   LOCAL bKeyBlock
   LOCAL cOldScr

   // Check for SET KEY first
   IF !( ( bKeyBlock := setkey( nKey ) ) == NIL )
      GetDoSetKey( bKeyBlock, oGet )
      RETURN                           // NOTE
   ENDIF

   DO CASE
   CASE ( nKey == K_UP )
      oGet:exitState := GE_UP
   CASE ( nKey == K_SH_TAB )
      oGet:exitState := GE_UP
   CASE ( nKey == K_DOWN )
      oGet:exitState := GE_DOWN
   CASE ( nKey == K_TAB )
      oGet:exitState := GE_DOWN
   CASE ( nKey == K_ENTER )
      oGet:exitState := GE_ENTER
   CASE ( nKey == K_ESC )
      IF ( SET( _SET_ESCAPE ) )
         oGet:undo()
         oGet:exitState := GE_ESCAPE
      ENDIF
   CASE ( nKey == K_PGUP )
      oGet:exitState := GE_WRITE
   CASE ( nKey == K_PGDN )
      oGet:exitState := GE_WRITE
   CASE ( nKey == K_CTRL_HOME )
      oGet:exitState := GE_TOP
   // Both ^W and ^End terminate the READ (the default)
   CASE ( nKey == K_CTRL_W )
      oGet:exitState := GE_WRITE
   CASE ( nKey == K_INS )
      SET( _SET_INSERT, !SET( _SET_INSERT ) )
   CASE ( nKey == K_UNDO )
      oGet:undo()
   CASE ( nKey == K_HOME )
      oGet:home()
   CASE ( nKey == K_END )
      oGet:end()
   CASE ( nKey == K_RIGHT )
      oGet:right()
   CASE ( nKey == K_LEFT )
      oGet:left()
   CASE ( nKey == K_CTRL_RIGHT )
      oGet:wordRight()
   CASE ( nKey == K_CTRL_LEFT )
      oGet:wordLeft()
   CASE ( nKey == K_BS )
      oGet:backSpace()
   CASE ( nKey == K_DEL )
      oGet:delete()
   CASE ( nKey == K_CTRL_T )
      oGet:delWordRight()
   CASE ( nKey == K_CTRL_Y )
      oGet:delEnd()
   CASE ( nKey == K_CTRL_BS )
      oGet:delWordLeft()
   OTHERWISE
      IF nKey = 32 // spacebar pops up the memoedit()

         cOldScr := SAVESCREEN( oGet:cargo[CARGO_TOP],;
                            oGet:cargo[CARGO_LEFT],oGet:cargo[CARGO_BOTTOM],;
                            oGet:cargo[CARGO_RIGHT] )
         @ oGet:cargo[CARGO_TOP], oGet:cargo[CARGO_LEFT] ;
           CLEAR TO oGet:cargo[CARGO_BOTTOM], oGet:cargo[CARGO_RIGHT]
         @ oGet:cargo[CARGO_TOP], oGet:cargo[CARGO_LEFT] ;
           TO oGet:cargo[CARGO_BOTTOM], oGet:cargo[CARGO_RIGHT] DOUBLE

         MemoEdit( oGet:cargo[CARGO_MEMO],oGet:cargo[CARGO_TOP]+1            ,;
                 oGet:cargo[CARGO_LEFT]+1,oGet:cargo[CARGO_BOTTOM]-1         ,;
                 oGet:cargo[CARGO_RIGHT]-1, oGet:cargo[CARGO_EDITMODE]       ,;
                 oGet:cargo[CARGO_USERFUNC], oGet:cargo[CARGO_LINELEN]       ,;
                 oGet:cargo[CARGO_TABSIZE], oGet:cargo[CARGO_TEXTBUFFERROW]  ,;
                 oGet:cargo[CARGO_TEXTBUFFERCOL], oGet:cargo[CARGO_WINDOWROW],;
                 oGet:cargo[CARGO_WINDOWCOL])

         RESTSCREEN(  oGet:cargo[CARGO_TOP], oGet:cargo[CARGO_LEFT],    ;
                      oGet:cargo[CARGO_BOTTOM], oGet:cargo[CARGO_RIGHT],;
                      cOldScr )

         oGet:ExitState := GE_ENTER

      ENDIF

   ENDCASE

   RETURN

****************************** EOF MEMOREAD.PRG *******************************
